Method: Failbot#context

Defined in:
lib/failbot.rb

#contextObject

Stack of context information to include in the next failbot report. These hashes are condensed down into one and included in the next report. Don’t mess with this structure directly - use the #push and #pop methods.



209
210
211
# File 'lib/failbot.rb', line 209

def context
  @thread_local_context.value
end